package com.mxnavi.mlink.serversdk.service;

import android.graphics.Bitmap;
import android.os.Bundle;
import android.os.ParcelFileDescriptor;
import android.hardware.usb.UsbAccessory;
import android.content.Intent;
import com.mxnavi.mlink.serversdk.service.IMlinkServiceClient;

interface IMlinkService {
	/**
	 * Check if there is a client connected.
	 */
	boolean isMlinkClientConnected();

	oneway void onUsbAccessoryAcquired(in UsbAccessory accessory);
	oneway void onMediaProjectionAcquired(int resultCode, in Intent data);

	oneway void registerMlinkServiceClient(in IMlinkServiceClient client);
	oneway void unregisterMlinkServiceClient(in IMlinkServiceClient client);


	boolean sendMediaInit(int sampleRate, int channelConfig, int sampleFormat);
	boolean sendMediaPlay();
	boolean sendMediaPause();
	boolean sendMediaData(in byte[] buffer);
    boolean sendMediaInfo(String source, String song, String artist, String album,in byte[] albumArt,
                          int duration, int playlistNum, String songId, int mode);

    boolean sendNaviTTSInit(int sampleRate, int channelConfig, int sampleFormat);
    boolean sendNaviTTSData(in byte[] buffer);
    boolean sendNaviTTSEnd();

    boolean sendTelAudioInit(int sampleRate, int channelConfig, int sampleFormat);
    boolean sendTelAudioData(in byte[] data);
    boolean sendTelAudioStop();
    boolean sendTelAudioStopp(boolean bStayIn);
    boolean sendTelCallStateInfo(String phoneNum, String callerName, int status);

    /**
     * 通知车机端进入唤醒状态了，需要开始录音
     */
    boolean sendMicRecordWakeupStart();

    /**
     * 通知车机端进入识别状态了，需要开始录音
     */
    boolean sendMicRecordRecogStart();

    /**
     * 发送录音结束命令给车机
     */
    boolean sendMicRecordEnd();

    boolean sendVRAudioInit(int sampleRate, int channelConfig, int sampleFormat);
    boolean sendVRAudioData(in byte[] data);
    boolean sendVRAAudioStop();
    boolean sendVRAAudioStopp(boolean bStayIn);
    boolean sendVRAudioInterrupt();
    boolean sendVRModuleStatusMsg(int statusId);
    
    boolean sendNaviNextTurnInfo(int action, int nextTurn, String roadName, int totalDistance, int remainDistance, int time);

    boolean sendCarlifeModuleStatus(int moduleId, int statusId);

    String getLinkProtocol();

    boolean pauseMedia();
    boolean resumeMedia();
    boolean sendGoToDesktop();
    boolean useMobileMic();

    boolean sendBluetoothInfo(String name, String address);

    boolean onBtConnectionStatus(boolean isConnected, String mac);
    boolean onReceivedBtData(in byte[] data);
    
    boolean onWifiConnectionStatus(boolean isConnected, String mac);

    // 以下 api 仅供 CRV 车机使用
    /**
     * 设定context infomation函数，以下时机调用：
     * 1. QQMusic, 喜马拉雅开始播放
     * 2. Launcher， QQMusic，喜马拉雅，语助到前台
     * @param id appID 具体参考下表
     * lancher_appid = 0x1;
     * navi_appid = 0x2;
     * radio_appid = 0x3;
     * musci_appid = 0x4;
     * weather_appid = 0x5;
     * setting_appid = 0x6;
     * vr_appid = 0x7;
     * tencent tv = 0x8;
     */
    int setContextInfo(int id);
    int appStopped(int id);
}
